#
# rica_utils.rb
#   -- Ruby Irc Core Agents (based on RFC1459)
#   NISHI Takao <zophos@koka-in.org>
#
# $Id: $
#
require 'rica'

class CyclicArray<SimpleDelegator
    SIZE_UNLIMIT=-1

    def initialize(*bufsize)
	@log=Array.new
	
	@maxsize=SIZE_UNLIMIT
	@keepsize=SIZE_UNLIMIT
	case bufsize.size
	when 0
	    #
	when 1
	    self.setMaxSize(bufsize[0])
	else
	    self.setSize(bufsize[0],bufsize[1])
	end

	super(@log)
    end
    
    attr_reader :@maxsize
    attr_reader :@keepsize

    def setMaxSize(maxsize)
	self.setSize(maxsize,@keepsize)
    end

    def setKeepSize(keepsize)
	self.setSize(@maxsize,keepsize)
    end

    def setSize(maxsize,keepsize)
	@maxsize=maxsize.to_i
	@keepsize=keepsize.to_i
	if(@maxsize<0)
	    @maxsize=SIZE_UNLIMIT
	end

	if(@keepsize>@maxsize)
	    @keepsize=@maxsize
	end
    end

    def push(msg)
	@log.push(msg)
	if((@maxsize!=SIZE_UNLIMIT)&&(@log.size>@maxsize))
	    expire
	end
    end

    private
    def expire
	@log.fill(nil,0,@maxsize-@keepsize)
	@log.compact!
    end
end

class IrcUserAttr
    def initialize
	@nick=nil
	@login=nil
	@realname=nil
	@mode=nil
	@channels=Hash.new
    end
    attr_accessor :nick
    attr_accessor :login
    attr_accessor :realname
    attr_accessor :mode
    attr_accessor :channels
end

class IrcChannelAttr
    DEFAULT_LOG_MAX=500
    DEFAULT_LOG_KEEP=300

    def initialize(chnl)
	@channel=chnl
	@topic=""
	@mode=""
	@members=Hash.new
	@log=CyclicArray.new(DEFAULT_LOG_MAX,DEFAULT_LOG_KEEP)
    end
    

    def setMode(msg)

    end

    def setTopic(msg)
	
    end

    def setMembers(msg)
	
    end

    def addMember(msg)
	
    end

    def delMember(msg)

    end

    def changeNick(msg)

    end
end

############################################################
#
# Channel buffer class
#
# Sample code of Rica sub class.
#
class IrcChannelBuffer<Rica
    def initialize
	super()
    end
end


############################################################
#
# Ctcp responcer class
#
# Sample code of Rica sub class.
#
class IrcCtcpResponcer<Rica
    VERSION=RICA_NAME+" "+RICA_VERION+" on Ruby "+String(RUBY_VERSION)
    USERINFO=[RICA_FULLNAME,RICA_INFO_URL]
    CLIENTINFO="ACTION CLIENTINFO ECHO PING TIME USERINFO VERSION"

    def initialize
	super()
	@clientinfo=CLIENTINFO
	@userinfo=USERINFO
	@version=VERSION
    end

    attr_accessor :userinfo
    attr_accessor :version
    
    def update(msg)
	case msg.command
	when RECV_CMND_CTCP_QUERY_CLIENTINFO
	    ctcp_answer_clientinfo(msg.server,msg.fromNick,@clientinfo)
	when RECV_CMND_CTCP_QUERY_ECHO
	    ctcp_answer_echo(msg.server,msg.fromNick,msg.args[0])
	when RECV_CMND_CTCP_QUERY_PING
	    ctcp_answer_ping(msg.server,msg.fromNick,msg.args[0])
	when RECV_CMND_CTCP_QUERY_TIME
	    ctcp_answer_time(msg.server,msg.fromNick)
	when RECV_CMND_CTCP_QUERY_USERINFO
	    ctcp_answer_userinfo(msg.server,msg.fromNick,@userinfo)
	when RECV_CMND_CTCP_QUERY_VERSION
	    ctcp_answer_version(msg.server,msg.fromNick,@version)
	end
    end
end


############################################################
#
# appendix
#
# 4.4BSD daemon() method
#
def daemon(nochdir,noclose)
    
    catch(:BEDAEMON) do
        unless fork()
            Process::setsid
	    
            unless fork()
                unless(nochdir)
		    Dir::chdir("/")
                end
		
                unless(noclose)
                    STDIN.close
                    STDOUT.close
                    STDERR.close
                end
		
                throw :BEDAEMON
            end
	end
	exit!
    end

end
